﻿//////////////////////////////////////////////
// BlendStateManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class BlendState ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT BlendStateManager : public nkCommon::SingletonClass<BlendStateManager>
	{
		friend class nkCommon::SingletonClass<BlendStateManager> ;

		private :

			// Attributs
			// Liste
			std::unordered_map<std::string, BlendState*> _memory ;

			// Default state
			BlendState* _defaultState ;

			// Threading
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur
			BlendStateManager () ;
		
		public :

			// Destructeur
			~BlendStateManager () ;
		
			// Manipulation mémoire
			BlendState* createOrRetrieve (const nkMemory::StringView& name) ;
			BlendState* get (const nkMemory::StringView& name) ;
			BlendState* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Getters
			BlendState* getDefaultState () const ;
	} ;
}